import 'package:cramr_ui_lib/model/practice_session.dart';
import 'package:cramr_ui_lib/model/study_mode.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_study_cover_mode_route.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_study_flashcard_mode_route.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_study_write_mode_route.dart';
import 'package:cramr_ui_lib/settings/settings_controller.dart';
import 'package:cramr_ui_lib/settings/settings_view.dart';
import 'package:cramr_ui_lib/util/text_size_limiter.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

import '../model/verb.dart';
import '../model/verb_test_model.dart';
import '../theme/custom_theme.dart';

/// The route for selecting the Study Mode
class VerbStudymodeSelectionRoute extends StatelessWidget {
  static const routeName = '/verbStudymodeSelectionRoute';

  final SettingsController _settingsController;

  const VerbStudymodeSelectionRoute(this._settingsController, {Key? key})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
        animation: _settingsController,
        builder: (BuildContext context, Widget? child) {
          return MaterialApp(
            debugShowCheckedModeBanner: false,
            themeMode: _settingsController.themeMode,
            theme: CustomTheme.lightTheme,
            darkTheme: ThemeData.dark(),
            home: SafeArea(
                child: Scaffold(
              appBar: AppBar(
                leading: IconButton(
                    key: const Key('BackKey'),
                    onPressed: () {
                      Navigator.pop(context);
                    },
                    icon: const Icon(Icons.arrow_back)),
                actions: [_settingsIcon(context)],
              ),
              body: Center(
                key: const Key('StudyModeSelection'),
                child: ListView(
                  children: [
                    InkWell(
                      key: const Key('buttonCoverMode'),
                      child: _buildCard(
                          context, 'afdekken', Icons.view_headline_sharp),
                      onTap: () {
                        _launchNextRoute(context, StudyMode.coverMode);
                      },
                    ),
                    InkWell(
                      key: const Key('buttonFlashCardMode'),
                      child:
                          _buildCard(context, 'kaartjes', Icons.style_outlined),
                      onTap: () {
                        _launchNextRoute(context, StudyMode.flashCardMode);
                      },
                    ),
                    InkWell(
                      key: const Key('buttonWriteMode'),
                      child: _buildCard(
                          context, 'schrijven', Icons.app_registration),
                      onTap: () {
                        _launchNextRoute(context, StudyMode.writtenMode);
                      },
                    )
                  ],
                ),
              ),
            )),
          );
        });
  }

  void _launchNextRoute(BuildContext context, StudyMode studyMode) {
    //start streak session
    final VerbTestModel selectedVerb =
        Provider.of<VerbTestModel<Verb>>(context, listen: false);
    _settingsController.streakEngine.addSession(PracticeSession(
        DateTime.now(),
        selectedVerb.mode,
        selectedVerb.tense,
        selectedVerb.items,
        _studyMethod(studyMode),
        selectedVerb.infinitive));
    Navigator.pushNamed(context, _routeName(studyMode));
  }

  Card _buildCard(BuildContext context, String modeName, IconData iconData) {
    return Card(
      margin: const EdgeInsets.fromLTRB(30, 15, 30, 15),
      elevation: 20,
      child: ListTile(
        key: UniqueKey(),
        title: Column(
          key: UniqueKey(),
          mainAxisAlignment: MainAxisAlignment.center,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            Padding(
              key: UniqueKey(),
              padding: const EdgeInsets.fromLTRB(10, 10, 10, 10),
              child: Text(
                modeName,
                key: UniqueKey(),
                style: Theme.of(context).textTheme.headline5,
                textScaleFactor: TextSizeLimiter.textScaleFactor(context),
              ),
            ),
            Padding(
              padding: const EdgeInsets.fromLTRB(10, 10, 10, 10),
              child: Icon(
                iconData,
                key: UniqueKey(),
                color: Theme.of(context).textTheme.headline1!.color,
                size: 50.0,
              ),
            ),
          ],
        ),
      ),
    );
  }

  String _routeName(StudyMode studyMode) {
    switch (studyMode) {
      case StudyMode.coverMode:
        return VerbIndicativeStudyCoverModeRoute.routeName;
      case StudyMode.flashCardMode:
        return VerbIndicativeStudyFlashcardModeRoute.routeName;
      case StudyMode.writtenMode:
        return VerbIndicativeStudyWriteModeRoute.routeName;
    }
  }

  StudyMethod _studyMethod(StudyMode studyMode) {
    switch (studyMode) {
      case StudyMode.coverMode:
        return StudyMethod.coverMethod;
      case StudyMode.flashCardMode:
        return StudyMethod.flashcardMethod;
      case StudyMode.writtenMode:
        return StudyMethod.writtenMethod;
    }
  }

  IconButton _settingsIcon(BuildContext context) {
    return IconButton(
        key: const Key('settingsButton'),
        icon: const Icon(Icons.settings),
        onPressed: () {
          Navigator.pushNamed(context, SettingsView.routeName);
        });
  }
}
