import 'dart:convert';

import 'package:cramr_ui_lib/model/practice_session.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:cramr_ui_lib/model/streak_engine.dart';

void main() {
  group('General tests for streak engine', () {
    test('simple test of StreakEngine', () {
      StreakEngine engine = StreakEngine();
      expect(engine.hasSessions(), false);
    });
    test('test number of sessions and add method', () {
      StreakEngine engine = StreakEngine();
      expect(engine.numberOfSessions(), 0);
      engine.addSession(PracticeSession(DateTime.now(), '_mode', '_tense', [],
          StudyMethod.flashcardMethod, 'infinitive'));
      expect(engine.numberOfSessions(), 1);
      engine.addSession(PracticeSession(DateTime.now(), '_mode', '_tense', [],
          StudyMethod.flashcardMethod, 'infinitive'));
      expect(engine.numberOfSessions(), 2);
    });
    test('test for presence of session on current day', () {
      StreakEngine engine = StreakEngine();
      var today = DateTime.now();
      engine.addSession(PracticeSession(today, '_mode', '_tense', [],
          StudyMethod.flashcardMethod, 'infinitive'));
      PracticeSession lastSession = engine.lastSession();
      var dateKey = lastSession.dateKey();
      expect(dateKey.year, today.year);
      expect(dateKey.month, today.month);
      expect(dateKey.day, today.day);
    });
    test('test for three day streak until today', () {
      var oneDay = const Duration(days: 1);
      var today = DateTime.now();
      var yesterday = today.subtract(oneDay);
      var dayBeforeYesterday = yesterday.subtract(oneDay);
      StreakEngine engine = StreakEngine();
      engine.addSession(PracticeSession(dayBeforeYesterday, '_mode', '_tense',
          [], StudyMethod.flashcardMethod, 'infinitive'));
      engine.addSession(PracticeSession(yesterday, '_mode', '_tense', [],
          StudyMethod.flashcardMethod, 'infinitive'));
      engine.addSession(PracticeSession(today, '_mode', '_tense', [],
          StudyMethod.writtenMethod, 'infinitive'));
      expect(engine.hasValidStreak(), true);
      expect(engine.streakCount(), 3);
    });
    test('test for skipped day with hole of 1 day', () {
      var oneDay = const Duration(days: 1);
      var today = DateTime.now();
      var yesterday = today.subtract(oneDay);
      var dayBeforeYesterday = yesterday.subtract(oneDay);
      StreakEngine engine = StreakEngine();
      engine.addSession(PracticeSession(dayBeforeYesterday, '_mode', '_tense',
          [], StudyMethod.coverMethod, 'infinitive'));
      engine.addSession(PracticeSession(today, '_mode', '_tense', [],
          StudyMethod.flashcardMethod, 'infinitive'));
      expect(engine.hasValidStreak(), false);
      expect(engine.streakCount(), 1);
    });
    test('test for gap of several days', () {
      var oneDay = const Duration(days: 1);
      var today = DateTime.now();
      var yesterday = today.subtract(oneDay);
      var dayBeforeYesterday = yesterday.subtract(oneDay);
      var threeDaysAgo = dayBeforeYesterday.subtract(oneDay);
      var fourDaysAgo = threeDaysAgo.subtract(oneDay);
      StreakEngine engine = StreakEngine();
      engine.addSession(PracticeSession(fourDaysAgo, '_mode', '_tense', [],
          StudyMethod.flashcardMethod, 'infinitive'));
      engine.addSession(PracticeSession(today, '_mode', '_tense', [],
          StudyMethod.writtenMethod, 'infinitive'));
      expect(engine.hasValidStreak(), false);
      expect(engine.streakCount(), 1);
    });
    test('find sessions for given date', () {
      StreakEngine engine = StreakEngine();
      DateTime today = DateTime.now();
      engine.addSession(PracticeSession(today, '_mode', '_tense', [],
          StudyMethod.writtenMethod, 'infinitive'));
      engine.addSession(PracticeSession(
          today, '_mode', '_tense', [], StudyMethod.coverMethod, 'infinitive'));
      DateKey expectedDateKey = DateKey(today.year, today.month, today.day);
      expect(engine.sessionsForDateKey(expectedDateKey).length, 2);
    });
    test('test the JSON methods', () {
      StreakEngine engine = StreakEngine();
      DateTime today = DateTime.now();
      engine.addSession(PracticeSession(today, '_mode', '_tense', [],
          StudyMethod.writtenMethod, 'infinitive'));
      String encoded = jsonEncode(engine);
      var decodeMap = jsonDecode(encoded);
      StreakEngine parsed = StreakEngine.fromJson(decodeMap);
      expect(parsed.sessions.length, engine.sessions.length);
    });
  });
  group('sessionsPerInfinitive method tests', () {
    test('test sessionsPerInfinitive method with infinitive', () {
      StreakEngine engine = StreakEngine();
      String infinitive1 = 'infinitive1';
      String infinitive2 = 'infinitive2';
      String infinitive3 = 'infinitive3';
      populateEngineWithNullableInfinitives(
          engine, infinitive1, infinitive2, infinitive3);
      expect(engine.sessionConjugationsPerInfinitive().length, 3);
      expect(engine.sessionConjugationsPerInfinitive()[infinitive1]?.length, 1);
      expect(engine.sessionConjugationsPerInfinitive()[infinitive2]?.length, 2);
      expect(engine.sessionConjugationsPerInfinitive()[infinitive3]?.length, 3);
    });
    test('test sessionsPerInfinitive method without infinitive', () {
      StreakEngine engine = StreakEngine();
      populateEngineWithNullableInfinitives(engine, null, null, null);
      expect(engine.sessionConjugationsPerInfinitive().length, 0);
    });
    test('test sessionsPerInfinitive method with and without infinitives', () {
      StreakEngine engine = StreakEngine();
      String infinitive1 = 'infinitive1';
      String infinitive2 = 'infinitive2';
      String infinitive3 = 'infinitive3';
      populateEngineWithNullableInfinitives(
          engine, infinitive1, null, infinitive3);
      expect(engine.sessionConjugationsPerInfinitive().length, 2);
      expect(engine.sessionConjugationsPerInfinitive()[infinitive1]?.length, 1);
      expect(
          engine.sessionConjugationsPerInfinitive()[infinitive2]?.length, null);
      expect(engine.sessionConjugationsPerInfinitive()[infinitive3]?.length, 3);
    });
  });
}

void populateEngineWithNullableInfinitives(StreakEngine engine,
    String? infinitive1, String? infinitive2, String? infinitive3) {
  DateTime today = DateTime.now();
  var yesterday = today.subtract(const Duration(days: 1));
  String mode1 = 'mode1';
  String mode2 = 'mode2';
  String mode3 = 'mode3';
  String tense1 = 'tense1';
  String tense2 = 'tense2';
  String tense3 = 'tense3';
  PracticeSession practiceSession1 = PracticeSession(
      today, mode1, tense2, [], StudyMethod.writtenMethod, infinitive3);
  PracticeSession practiceSession2 = PracticeSession(
      today, mode2, tense3, [], StudyMethod.writtenMethod, infinitive1);
  PracticeSession practiceSession3 = PracticeSession(
      today, mode3, tense2, [], StudyMethod.writtenMethod, infinitive2);
  PracticeSession practiceSession4 = PracticeSession(
      today, mode1, tense2, [], StudyMethod.writtenMethod, infinitive3);
  PracticeSession practiceSession5 = PracticeSession(
      today, mode2, tense3, [], StudyMethod.writtenMethod, infinitive1);
  PracticeSession practiceSession6 = PracticeSession(
      today, mode3, tense1, [], StudyMethod.writtenMethod, infinitive2);
  PracticeSession practiceSession7 = PracticeSession(
      today, mode1, tense2, [], StudyMethod.writtenMethod, infinitive3);
  PracticeSession practiceSession8 = PracticeSession(
      today, mode2, tense3, [], StudyMethod.writtenMethod, infinitive1);
  PracticeSession practiceSession9 = PracticeSession(
      today, mode3, tense1, [], StudyMethod.writtenMethod, infinitive2);
  PracticeSession practiceSession10 = PracticeSession(
      yesterday, mode1, tense1, [], StudyMethod.writtenMethod, infinitive3);
  PracticeSession practiceSession12 = PracticeSession(
      yesterday, mode2, tense3, [], StudyMethod.writtenMethod, infinitive1);
  PracticeSession practiceSession13 = PracticeSession(
      yesterday, mode3, tense1, [], StudyMethod.writtenMethod, infinitive2);
  PracticeSession practiceSession14 = PracticeSession(
      yesterday, mode1, tense2, [], StudyMethod.writtenMethod, infinitive3);
  PracticeSession practiceSession15 = PracticeSession(
      yesterday, mode2, tense3, [], StudyMethod.writtenMethod, infinitive1);
  PracticeSession practiceSession16 = PracticeSession(
      yesterday, mode3, tense1, [], StudyMethod.writtenMethod, infinitive2);
  PracticeSession practiceSession17 = PracticeSession(
      yesterday, mode1, tense3, [], StudyMethod.writtenMethod, infinitive3);
  PracticeSession practiceSession18 = PracticeSession(
      yesterday, mode2, tense3, [], StudyMethod.writtenMethod, infinitive1);
  PracticeSession practiceSession19 = PracticeSession(
      yesterday, mode3, tense1, [], StudyMethod.writtenMethod, infinitive2);
  engine.addSession(practiceSession1);
  engine.addSession(practiceSession2);
  engine.addSession(practiceSession3);
  engine.addSession(practiceSession4);
  engine.addSession(practiceSession5);
  engine.addSession(practiceSession6);
  engine.addSession(practiceSession7);
  engine.addSession(practiceSession8);
  engine.addSession(practiceSession9);
  engine.addSession(practiceSession10);
  engine.addSession(practiceSession12);
  engine.addSession(practiceSession13);
  engine.addSession(practiceSession14);
  engine.addSession(practiceSession15);
  engine.addSession(practiceSession16);
  engine.addSession(practiceSession17);
  engine.addSession(practiceSession18);
  engine.addSession(practiceSession19);
}
